/****************************************************************************/
/* T  vF SD J[hWX^CAEg										*/
/*   F SDJ[h̃WX^CAEg`							*/
/* 쐬F 2004.10.31														*/
/* XVF 2013.09.30														*/
/* Copyright    resove														*/
/****************************************************************************/
#ifndef DRV_SD_REGISTER_H
#define DRV_SD_REGISTER_H

/****************************************************************************/
/*	CSD ̃tH[}bg														*/
/*               +----------------------------------------- a:CSD structure. 0:Version 1.0	*/
/*               |    +------------------------------------ b:spec version					*/
/*               |    | +---------------------------------- -:reserved						*/
/*               |    | |      +--------------------------- c:data read access time-1(TAAC)	*/
/*               |    | |      |                              detail						*/
/*               |    | |      |                              [2:0] time unit. 0:1ns, 1:10ns, 2:100ns, 3:1us, 4:10us, 5:100us, 6:1ms, 7:10ms	*/
/*               |    | |      |                              [6:3] time value. 0:reserved, 1:1.0, 2:1.2, 3:1.3, 4:1.5, 5:2.0, 6:2.5, 7:3.0, 8:3.5, 9:4.0Ca:4.5, b:5.0, c:5.5, d:6.0, e:7.0, f:8.0	*/
/*               |    | |      |                              [7:7] reserved									*/
/*               |    | |      |         +----------------- d:data read access time-1 in CLK cycle(NSAC*100)	*/
/*               |    | |      |         |         +------- e:max. data transfer rate							*/
/*               |    | |      |         |         |          detail											*/
/*               |    | |      |         |         |          [2:0] transfer rate unit. 0:100kbps, 1:1Mbps, 2:10Mbps, 3:100Mbps, 4-7:reserved	*/
/*               |    | |      |         |         |          [6:3] time value. 0:reserved, 1:1.0, 2:1.2, 3:1.3,	*/
/*               |    | |      |         |         |                            4:1.5, 5:2.0, 6:2.5, 7:3.0, 8:3.5,	*/
/*               |    | |      |         |         |                            9:4.0Ca:4.5, b:5.0, c:5.5, d:6.0,	*/
/*               |    | |      |         |         |                            e:7.0, f:8.0						*/
/*               |    | |      |         |         |          [7:7] reserved		*/
/*               |    | |      |         |         |						*/
/*              +---------+---------+---------+---------+					*/
/*              |aabb bb--|cccc cccc|dddd dddd|eeee eeee|					*/
/*              +---------+---------+---------+---------+					*/
/*             +0        +1        +2        +3          (Byte ʒu)		*/
/*																			*/
/*               +----------------------------------------- f:card command classes				*/
/*               |               +------------------------- g:max. read data block length		*/
/*               |               |   +--------------------- h:partial blocks for read allow		*/
/*               |               |   |+-------------------- i:write block misalignment			*/
/*               |               |   ||+------------------- j:read block misalignment			*/
/*               |               |   |||+------------------ k:DSR implemented					*/
/*               |               |   |||| +---------------- l:reserved							*/
/*               |               |   |||| |       +-------- n:device size						*/
/*               |               |   |||| |       |							*/
/*              +---------+---------+---------+---------+					*/
/*              |ffff ffff|ffff gggg|hijk llnn|nnnn nnnn|					*/
/*              +---------+---------+---------+---------+					*/
/*             +4        +5        +6        +7								*/
/*																			*/
/*               +----------------------------------------- n:device size	*/
/*               |   +------------------------------------- o:max. read current @Vdd min	*/
/*               |   |  +---------------------------------- p:max. read current @Vdd max	*/
/*               |   |  |   +------------------------------ q:max. write current @Vdd min	*/
/*               |   |  |   |   +-------------------------- r:max. write current @Vdd max	*/
/*               |   |  |   |   |  +----------------------- s:device size multiplire		*/
/*               |   |  |   |   |  |    +------------------ t:erase group size				*/
/*               |   |  |   |   |  |    |      +----------- u:erase group size multi		*/
/*               |   |  |   |   |  |    |      |     +----- v:write protect group size		*/
/*               |   |  |   |   |  |    |      |     |						*/
/*              +---------+---------+---------+---------+					*/
/*              |nnoo oppp|qqqr rrss|sttt ttuu|uuuv vvvv|					*/
/*              +---------+---------+---------+---------+					*/
/*             +8        +9        +10       +11       						*/
/*																			*/
/*               +----------------------------------------- w:write protect group enable	*/
/*               | +--------------------------------------- x:default ECC					*/
/*               | |  +------------------------------------ y:write speed factor			*/
/*               | |  |   +-------------------------------- z:max.write data block length	*/
/*               | |  |   |  +----------------------------- A:partial blocks for write allowd	*/
/*               | |  |   |  |   +------------------------- B:reserved						*/
/*               | |  |   |  |   |   +--------------------- C:File format group				*/
/*               | |  |   |  |   |   |+-------------------- D:copy flag(OTP)				*/
/*               | |  |   |  |   |   ||+------------------- E:permanent write protection	*/
/*               | |  |   |  |   |   |||+------------------ F:temporary write protection	*/
/*               | |  |   |  |   |   |||| +---------------- G:File format					*/
/*               | |  |   |  |   |   |||| |  +------------- H:ECC							*/
/*               | |  |   |  |   |   |||| |  |    +-------- I:CRC							*/
/*               | |  |   |  |   |   |||| |  |    |    +--- J:not used,always '1'			*/
/*               | |  |   |  |   |   |||| |  |    |    |									*/
/*              +---------+---------+---------+---------+					*/
/*              |wxxy yyzz|zzAB BBBB|CDEF GGHH|IIII IIIJ|					*/
/*              +---------+---------+---------+---------+					*/
/*             +12       +13       +14       +15							*/
/*																			*/
/****************************************************************************/

#define SD_CSD_LEN		( 16 )						/* CSD 			*/

typedef union u_csd {
	unsigned char byte[SD_CSD_LEN];
	struct {
		unsigned char	structure:2;			/* [127:126]		*/
		unsigned char	specVersion:4;			/* [125:122]		*/
		unsigned char	RES_0:2;				/* [121:120]		*/
		unsigned char	RES_1:1;				/* [119:119]		*/
		unsigned char	taacValue:4;			/* [118:115]		*/
		unsigned char	taacUnit:3;				/* [114:112]		*/
		unsigned char	nsac:8;					/* [111:104]		*/
		unsigned char	RES_3:1;				/* [103:103]		*/
		unsigned char	rateValue:4;			/* [102:99]			*/
		unsigned char	rateUnit:3;				/* [98:96]			*/
		unsigned char	commandClass0:8;		/* [95:88]			*/
		unsigned char	commandClass1:4;		/* [87:84]			*/
		unsigned char	blockLength:4;			/* [83:80]			*/
		unsigned char	readBlockPartial:1;		/* [79:79]			*/
		unsigned char	writeMisalign:1;		/* [78:78]			*/
		unsigned char	readMisalign:1;			/* [77:77]			*/
		unsigned char	dsr:1;					/* [76:76]			*/
		unsigned char	RES_4:2;				/* [75:74]			*/
		unsigned char	cSize0:2;				/* [73:72]			*/
		unsigned char	cSize1:8;				/* [71:64]			*/
		unsigned char	cSize2:2;				/* [63:62]			*/
		unsigned char	readVddMin:3;			/* [61:59]			*/
		unsigned char	readVddMax:3;			/* [58:56]			*/
		unsigned char	writeVddMin:3;			/* [55:53]			*/
		unsigned char	writeVddMax:3;			/* [52:50]			*/
		unsigned char	cSizeMult0:2;			/* [49:48]			*/
		unsigned char	cSizeMult1:1;			/* [47:47]			*/
		unsigned char	eraseGroupSize:5;		/* [46:42]			*/
		unsigned char	eraseGroupMult0:2;		/* [41:40]			*/
		unsigned char	eraseGroupMult1:3;		/* [39:37]			*/
		unsigned char	writeProtectGroupSize:5;/* [36:32]			*/
		unsigned char	writeProtectGroupEna:1;	/* [31:31]			*/
		unsigned char	defaultEcc:2;			/* [30:29]			*/
		unsigned char	writeSpeedFactor:3;		/* [28:26]			*/
		unsigned char	maxWriteDataBlockSize0:2;/* [25:24]			*/
		unsigned char	maxWriteDataBlockSize1:2;/* [23:22]			*/
		unsigned char	writeBlockPartial:1;	/* [21:21]			*/
		unsigned char	RES_5:5;				/* [20:16]			*/
		unsigned char	fileFormatGroup:1;		/* [15:15]			*/
		unsigned char	copyFlag:1;				/* [14:14]			*/
		unsigned char	permWriteProtect:1;		/* [13:13]			*/
		unsigned char	tempWriteProtect:1;		/* [12:12]			*/
		unsigned char	fileFormat:2;			/* [11:10]			*/
		unsigned char	ecc:2;					/* [9:8]			*/
		unsigned char	crc7:7;					/* [7:1]			*/
		unsigned char	always1:1;				/* [0:0]			*/
	} field;
} T_SD_CSD;

/****************************************************************************/
/*	CID ̃tH[}bg														*/
/*               +------------------------------------------------ A:Manufacture ID			*/
/*               |                                                    0A1A...				*/
/*               |    +------------------------------------------- B:OEM/Applicatuon ID		*/
/*               |    |                                               "TM"A"PA"...			*/
/*               |    |        +---------------------------------- C:Product name			*/
/*               |    |        |          +----------------------- D:Product revision		*/
/*               |    |        |          |      +---------------- E:Product serial number	*/
/*               |    |        |          |      |        +------- F:Manufactureing date	*/
/*               |    |        |          |      |        |          detail					*/
/*               |    |        |          |      |        |          +---------+---------+	*/
/*               |    |        |          |      |        |          |xxxx yyyy|yyyy mmmm|	*/
/*               |    |        |          |      |        |          +---------+---------+	*/
/*               |    |        |          |      |        |           xx: Reserved			*/
/*               |    |        |          |      |        |           yy: year code. 0=2000	*/
/*               |    |        |          |      |        |           mm: month code. 1=January	*/
/*               |    |        |          |      |        |    +-- G:CRC7 checksum			*/
/*               |    |        |          |      |        |    |     [7:1]CRC7 check sum	*/
/*               |    |        |          |      |        |    |     [0:0]not used,always'1'*/
/*               |    |        |          |      |        |    |							*/
/*              +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+							*/
/*              |AA|BB|BB|CC|CC|CC|CC|CC|DD|EE|EE|EE|EE|FF|FF|GG|							*/
/*              +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+							*/
/*             +0    +2    +4    +6    +8    +10   +12   +14  (Byte ʒu)					*/
/****************************************************************************/
#define SD_CID_LEN		( 16 )						/* CID 			*/

typedef union u_cid {
	unsigned char byte[SD_CID_LEN];
	struct {
		unsigned char	manufactureID;			/* [127:8]			*/
		unsigned char	oemAppID[2];			/* [119:16]			*/
		unsigned char	productName[5];			/* [103:40]			*/
		unsigned char	productRevision;		/* [63:8]			*/
		unsigned char	serialNo[4];			/* [55:32]			*/
		unsigned char	RES_1:4;				/* [23:4]			*/
		unsigned char	dateYY0:4;				/* [19:4]			*/
		unsigned char	dateYY1:4;				/* [15:4]			*/
		unsigned char	dateMM:4;				/* [11:4]			*/
		unsigned char	crc7:7;					/* [7:1]			*/
		unsigned char	always1:1;				/* [0:0]			*/
	} field;
} T_SD_CID;

/****************************************************************************/
/*	OCR ̃tH[}bg														*/
/*               +------------------------------------------------ A:Card power up status bit.			*/
/*               |    +------------------------------------------- -:Reserved.							*/
/*               |    |                                               "TM" :						*/
/*               |    |        +---------------------------------- a`s T|[gd3.6`1.7V(0.1V)	*/
/*               |    |        |                    +------------- 1`5 T|[gd1.7`1.45V(0.05V)	*/
/*               |    |        |                    |      					*/
/*              +---------+---------+---------+---------+					*/
/*              |A--- ----|abcd efgh|ijkl mnop|qrs1 2345|					*/
/*              +---------+---------+---------+---------+					*/
/*             +0        +1        +2        +3         +4  (Byte ʒu)		*/
/****************************************************************************/
#define SD_OCR_LEN		( 4 )					/* OCR 			*/

#endif /* #ifndef DRV_SD_REGISTER_H */
